@@ -331,7 +331,7 @@ populate_ui (CEPageIP6 *self)
 	NMSettingIP6Config *setting = priv->setting;
 	GtkListStore *store;
 	GtkTreeIter model_iter;
-	int method = IP6_METHOD_IGNORE;
+	int method = IP6_METHOD_AUTO;
 	GString *string = NULL;
 	SetMethodInfo info;
 	const char *str_method;
@@ -341,6 +341,8 @@ populate_ui (CEPageIP6 *self)
 	gtk_combo_box_set_active (priv->method, 0);
 	str_method = nm_setting_ip6_config_get_method (setting);
 	if (str_method) {
+		if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_IGNORE))
+			method = IP6_METHOD_IGNORE;
 		if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_AUTO))
 			method = IP6_METHOD_AUTO;
 		if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_DHCP))
@@ -927,6 +929,7 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 CEPage *
 ce_page_ip6_new (NMConnection *connection,
                  GtkWindow *parent_window,
+                 NMClient *client,
                  const char **out_secrets_setting_name,
                  GError **error)
 {
@@ -937,6 +940,7 @@ ce_page_ip6_new (NMConnection *connection,
 	self = CE_PAGE_IP6 (ce_page_new (CE_TYPE_PAGE_IP6,
 	                                 connection,
 	                                 parent_window,
+	                                 client,
 	                                 UIDIR "/ce-page-ip6.ui",
 	                                 "IP6Page",
 	                                 _("IPv6 Settings")));
