@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 - 2010 Red Hat, Inc.
+ * (C) Copyright 2008 - 2011 Red Hat, Inc.
  */
 
 #include "config.h"
@@ -363,6 +363,7 @@ finish_setup (CEPageMobile *self, gpointer unused, GError *error, gpointer user_
 CEPage *
 ce_page_mobile_new (NMConnection *connection,
                     GtkWindow *parent_window,
+                    NMClient *client,
                     const char **out_secrets_setting_name,
                     GError **error)
 {
@@ -372,6 +373,7 @@ ce_page_mobile_new (NMConnection *connection,
 	self = CE_PAGE_MOBILE (ce_page_new (CE_TYPE_PAGE_MOBILE,
 	                                    connection,
 	                                    parent_window,
+	                                    client,
 	                                    UIDIR "/ce-page-mobile.ui",
 	                                    "MobilePage",
 	                                    _("Mobile Broadband")));
@@ -650,7 +652,11 @@ mobile_connection_new (GtkWindow *parent,
 	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 12, 12, 12, 12);
 	gtk_box_pack_start (GTK_BOX (content), alignment, TRUE, FALSE, 6);
 
+#if GTK_CHECK_VERSION (3,1,6)
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
 	hbox = gtk_hbox_new (FALSE, 6);
+#endif
 	gtk_container_add (GTK_CONTAINER (alignment), hbox);
 
 	image = gtk_image_new_from_icon_name ("nm-device-wwan", GTK_ICON_SIZE_DIALOG);
@@ -658,7 +664,11 @@ mobile_connection_new (GtkWindow *parent,
 	gtk_misc_set_padding (GTK_MISC (image), 0, 6);
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 6);
 
+#if GTK_CHECK_VERSION (3,1,6)
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, FALSE, 0);
 
 	label = gtk_label_new (_("Select the technology your mobile broadband provider uses.  If you are unsure, ask your provider."));
