@@ -189,10 +189,11 @@ confirm_setup (MobileWizard *self)
 {
 	GtkWidget *vbox, *label, *alignment, *pbox;
 
+#if GTK_CHECK_VERSION(3,1,6)
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
-
-	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 	label = gtk_label_new (_("Your mobile broadband connection is configured with the following settings:"));
 	gtk_widget_set_size_request (label, 500, -1);
@@ -229,7 +230,11 @@ confirm_setup (MobileWizard *self)
 
 	alignment = gtk_alignment_new (0, 0.5, 0, 0);
 	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 25, 0);
+#if GTK_CHECK_VERSION(3,1,6)
+        pbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	pbox = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_container_add (GTK_CONTAINER (alignment), pbox);
 	gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, FALSE, 0);
 
@@ -465,7 +470,11 @@ plan_setup (MobileWizard *self)
 	GtkWidget *vbox, *label, *alignment, *hbox, *image;
 	GtkCellRenderer *renderer;
 
+#if GTK_CHECK_VERSION(3,1,6)
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 
 	label = gtk_label_new_with_mnemonic (_("_Select your plan:"));
@@ -507,7 +516,11 @@ plan_setup (MobileWizard *self)
 	gtk_container_add (GTK_CONTAINER (alignment), self->plan_unlisted_entry);
 	gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3,1,6)
+        hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
 	hbox = gtk_hbox_new (FALSE, 6);
+#endif
 	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
@@ -717,7 +730,11 @@ providers_setup (MobileWizard *self)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
 
+#if GTK_CHECK_VERSION(3,1,6)
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 
 	self->providers_view_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Select your provider from a _list:"));
@@ -1054,7 +1071,11 @@ country_setup (MobileWizard *self)
 	GtkTreeSelection *selection;
 	GtkTreeIter unlisted_iter;
 
+#if GTK_CHECK_VERSION(3,1,6)
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 	label = gtk_label_new (_("Country or Region List:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -1343,7 +1364,11 @@ intro_setup (MobileWizard *self)
 	GtkCellRenderer *renderer;
 	char *s;
 
+#if GTK_CHECK_VERSION(3,1,6)
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 
 	label = gtk_label_new (_("This assistant helps you easily set up a mobile broadband connection to a cellular (3G) network."));
@@ -1358,7 +1383,11 @@ intro_setup (MobileWizard *self)
 
 	alignment = gtk_alignment_new (0, 0, 1, 0);
 	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 25, 25, 0);
+#if GTK_CHECK_VERSION(3,1,6)
+        info_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	info_vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_container_add (GTK_CONTAINER (alignment), info_vbox);
 	gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, FALSE, 6);
 
